@@ -241,6 +241,7 @@ class property:
             size=None,  # type: str | None
             copyable=False,  # type: bool
             type_class=None,  # type: Type[TypeHint] | None
+            lazy_loading_fn=None,  # type: str | None
     ):  # type: (...) -> None
         """
         |UDM| property.
@@ -272,6 +273,7 @@ class property:
         :param size: The |UMC| widget size; one of :py:data:`univention.admin.syntax.SIZES`.
         :param copyable: With `True` the property is copied when the object is cloned; with `False` the new object will use the default value.
         :param type_class: An optional Typing class which overwrites the syntax class specific type.
+        :param lazy_loading_fn: An optional function name that implements loading additional expensive properties if requested.
         """
         self.short_description = short_description
         self.long_description = long_description
@@ -305,6 +307,7 @@ class property:
         self.size = size
         self.copyable = copyable
         self.type_class = type_class
+        self.lazy_loading_fn = lazy_loading_fn
 
     def new(self):
         # type: () -> list[str] | None
@@ -390,6 +393,10 @@ class property:
             return True
         return bool(set(self.options).intersection(set(options)))
 
+    def lazy_load(self, obj):
+        if self.lazy_loading_fn:
+            getattr(obj, self.lazy_loading_fn)()
+
 
 class option(object):
     """|UDM| option to make properties conditional."""
@@ -620,14 +627,18 @@ class policiesGroup:
 def _ldap_cache(ttl=10, cache_none=True):
     def _decorator(func):
         func._cache = {}
+        func._last_called = None
 
         def _decorated(lo, *args):
             cache = func._cache
             key = tuple([id(lo)] + list(args))
             now = time.time()
-            for cache_key, cache_val in list(cache.items()):
-                if cache_val['expire'] < now:
-                    cache.pop(cache_key)
+
+            if func._last_called and (now - func._last_called) > ttl:
+                for cache_key, cache_val in list(cache.items()):
+                    if cache_val['expire'] < now:
+                        cache.pop(cache_key, True)
+            func._last_called = now
 
             if key not in cache or cache[key]['expire'] < now:
                 value = {'value': func(lo, *args), 'expire': time.time() + ttl}
@@ -635,7 +646,14 @@ def _ldap_cache(ttl=10, cache_none=True):
                     return
                 cache[key] = value
             return cache[key]['value']
+
+        def cache_clear():
+            func._cache = {}
+
+        _decorated.cache_clear = cache_clear
+
         return _decorated
+
     return _decorator
 
 
