@@ -36,7 +36,7 @@ from __future__ import absolute_import
 
 import time
 from logging import getLogger
-from typing import Any, Callable, Dict, List, Optional, Tuple  # noqa: F401
+from typing import Any, Callable  # noqa: F401
 
 import ldap
 
@@ -125,7 +125,7 @@ class DN(object):
 
 
 def getBaseDN(host='localhost', port=None, uri=None):
-    # type: (str, Optional[int], Optional[str]) -> str
+    # type: (str, int | None, str | None) -> str
     """
     Return the naming context of the LDAP server.
 
@@ -133,7 +133,6 @@ def getBaseDN(host='localhost', port=None, uri=None):
     :param int port: The TCP port number of the LDAP server.
     :param str uri: A complete LDAP URI.
     :returns: The distinguished name of the LDAP root.
-    :rtype: str
     """
     if not uri:
         if not port:
@@ -150,33 +149,29 @@ def getBaseDN(host='localhost', port=None, uri=None):
     return result[0][1]['namingContexts'][0].decode('utf-8')
 
 
-def getAdminConnection(start_tls=2, decode_ignorelist=[]):
-    # type: (int, List[str]) -> Tuple[univention.admin.uldap.access, univention.admin.uldap.position]
+def getAdminConnection(start_tls=None, decode_ignorelist=[]):
+    # type: (int | None, list[str]) -> tuple[univention.admin.uldap.access, univention.admin.uldap.position]
     """
     Open a LDAP connection using the admin credentials.
 
     :param int start_tls: Negotiate TLS with server. If `2` is given, the command will require the operation to be successful.
     :param decode_ignorelist: List of LDAP attribute names which shall be handled as binary attributes.
-    :type decode_ignorelist: list[str]
     :return: A 2-tuple (LDAP-access, LDAP-position)
-    :rtype: tuple[univention.admin.uldap.access, univention.admin.uldap.position]
     """
     lo = univention.uldap.getAdminConnection(start_tls, decode_ignorelist=decode_ignorelist)
     pos = position(lo.base)
     return access(lo=lo), pos
 
 
-def getMachineConnection(start_tls=2, decode_ignorelist=[], ldap_master=True):
-    # type: (int, List[str], bool) -> Tuple[univention.admin.uldap.access, univention.admin.uldap.position]
+def getMachineConnection(start_tls=None, decode_ignorelist=[], ldap_master=True):
+    # type: (int | None, list[str], bool) -> tuple[univention.admin.uldap.access, univention.admin.uldap.position]
     """
     Open a LDAP connection using the machine credentials.
 
     :param int start_tls: Negotiate TLS with server. If `2` is given, the command will require the operation to be successful.
     :param decode_ignorelist: List of LDAP attribute names which shall be handled as binary attributes.
-    :type decode_ignorelist: list[str]
     :param bool ldap_master: Open a connection to the Primary if True, to the preferred LDAP server otherwise.
     :return: A 2-tuple (LDAP-access, LDAP-position)
-    :rtype: tuple[univention.admin.uldap.access, univention.admin.uldap.position]
     """
     lo = univention.uldap.getMachineConnection(start_tls, decode_ignorelist=decode_ignorelist, ldap_master=ldap_master)
     pos = position(lo.base)
@@ -190,7 +185,6 @@ def _err2str(err):
 
     :param Exception err: An exception instance.
     :returns: A concatenated string formatted from the exception
-    :rtype: str
     """
     msgs = []
     for iarg in err.args:
@@ -219,12 +213,11 @@ class domain(object):
         self.domain = self.lo.get(self.position.getDomain(), attr=['sambaDomain', 'sambaSID', 'krb5RealmName'])
 
     def getKerberosRealm(self):
-        # type: () -> Optional[str]
+        # type: () -> str | None
         """
         Return the name of the Kerberos realms.
 
         :returns: The name of the Kerberos realm.
-        :rtype: str
         """
         if 'krb5RealmName' not in self.domain:
             return None
@@ -280,7 +273,6 @@ class position(object):
         Return the distinguished name.
 
         :returns: The absolute DN.
-        :rtype: str
         """
         return ldap.dn.dn2str(ldap.dn.str2dn(self.__pos) + ldap.dn.str2dn(self.__base))
 
@@ -304,7 +296,6 @@ class position(object):
         Return the distinguished name relative to the LDAP base.
 
         :returns: The relative DN.
-        :rtype: str
         """
         return ldap.dn.explode_rdn(self.getDn())[0]
 
@@ -314,7 +305,6 @@ class position(object):
         Return the LDAP base DN.
 
         :returns: The distinguished name of the LDAP base.
-        :rtype: str
         """
         return self.__base
 
@@ -324,7 +314,6 @@ class position(object):
         Check if the position equals the LDAP base DN.
 
         :returns: True if the position equals the base DN, False otherwise.
-        :rtype: bool
         """
         return access.compare_dn(self.getDn(), self.getBase())
 
@@ -334,7 +323,6 @@ class position(object):
         Return the distinguished name of the domain part of the position.
 
         :returns: The distinguished name.
-        :rtype: str
         """
         if not self.__indomain or self.getDn() == self.getBase():
             return self.getBase()
@@ -351,7 +339,6 @@ class position(object):
         Return the distinguished name of the configuration container.
 
         :returns: The distinguished name.
-        :rtype: str
         """
         return u'cn=univention,' + self.getDomain()
 
@@ -361,7 +348,6 @@ class position(object):
         Check if the position equals the domain DN.
 
         :returns: True if the position equals the domain DN, False otherwise.
-        :rtype: bool
         """
         return self.getDn() == self.getDomain()
 
@@ -371,7 +357,6 @@ class position(object):
         Return the login domain name.
 
         :returns: The login domain name.
-        :rtype: str
         """
         return self.__loginDomain
 
@@ -381,7 +366,6 @@ class position(object):
         Switch position to parent container.
 
         :returns: False if already at the Base, True otherwise.
-        :rtype: bool
         """
         if self.isBase():
             return False
@@ -394,12 +378,11 @@ class access(object):
 
     @property
     def binddn(self):
-        # type: () -> Optional[str]
+        # type: () -> str | None
         """
         Return the distinguished name of the account.
 
         :returns: The distinguished name of the account (or `None` with |SAML|).
-        :rtype: str
         """
         return self.lo.binddn
 
@@ -410,7 +393,6 @@ class access(object):
         Return the user password or credentials.
 
         :returns: The user password or credentials.
-        :rtype: str
         """
         return self.lo.bindpw
 
@@ -421,7 +403,6 @@ class access(object):
         Return the host name of the LDAP server.
 
         :returns: the host name of the LDAP server.
-        :rtype: str
         """
         return self.lo.host
 
@@ -432,7 +413,6 @@ class access(object):
         Return the TCP port number of the LDAP server.
 
         :returns: the TCP port number of the LDAP server.
-        :rtype: int
         """
         return self.lo.port
 
@@ -443,7 +423,6 @@ class access(object):
         Return the LDAP base of the LDAP server.
 
         :returns: the LDAP base of the LDAP server.
-        :rtype: str
         """
         return self.lo.base
 
@@ -452,18 +431,28 @@ class access(object):
         # type: () -> int
         return self.lo.start_tls
 
-    def __init__(self, host='localhost', port=None, base=u'', binddn=u'', bindpw=u'', start_tls=2, lo=None, follow_referral=False, uri=None):
-        # type: (str, int, str, str, str, int, univention.uldap.access, bool) -> None
+    def __init__(
+        self,
+        host='localhost',  # type: str
+        port=None,  # type: int | None
+        base=u'',  # type: str
+        binddn=u'',  # type: str
+        bindpw=u'',  # type: str
+        start_tls=None,  # type: int | None
+        lo=None,  # type: univention.uldap.access | None
+        follow_referral=False,  # type: bool
+        uri=None,  # type: str | None
+    ):  # type: (...) -> None
         """
-        :param str host: The hostname of the |LDAP| server.
-        :param int port: The |TCP| port number of the |LDAP| server.
-        :param str base: The base distinguished name.
-        :param str binddn: The distinguished name of the account.
-        :param str bindpw: The user password for simple authentication.
-        :param int start_tls: Negotiate |TLS| with server. If `2` is given, the command will require the operation to be successful.
-        :param univention.uldap.access lo: |LDAP| connection.
-        :param:bool follow_referral: Follow |LDAP| referrals.
-        :param str uri: LDAP connection string.
+        :param host: The hostname of the |LDAP| server.
+        :param port: The |TCP| port number of the |LDAP| server.
+        :param base: The base distinguished name.
+        :param binddn: The distinguished name of the account.
+        :param bindpw: The user password for simple authentication.
+        :param start_tls: Negotiate |TLS| with server. If `2` is given, the command will require the operation to be successful.
+        :param lo: |LDAP| connection.
+        :param follow_referral: Follow |LDAP| referrals.
+        :param uri: LDAP connection string.
         """
         if lo:
             self.lo = lo
@@ -510,7 +499,7 @@ class access(object):
         self.__require_licence()
 
     def bind_oauthbearer(self, authzid, bindpw):
-        # type: (Optional[str], str) -> None
+        # type: (str | None, str) -> None
         """
         Do LDAP bind using OAuth 2.0 Access Token.
 
@@ -578,7 +567,6 @@ class access(object):
         Return the distinguished name of the authenticated user.
 
         :returns: The distinguished name.
-        :rtype: str
         """
         return self.lo.whoami()
 
@@ -603,7 +591,6 @@ class access(object):
         Retrieve |LDAP| schema information from |LDAP| server.
 
         :returns: The |LDAP| schema.
-        :rtype: ldap.schema.subentry.SubSchema
         """
         return self.lo.get_schema()
 
@@ -616,12 +603,11 @@ class access(object):
         :param str a: The first distinguished name.
         :param str b: A second distinguished name.
         :returns: True if the DNs are the same, False otherwise.
-        :rtype: bool
         """
         return univention.uldap.access.compare_dn(a, b)
 
     def get(self, dn, attr=[], required=False, exceptions=False):
-        # type: (str, List[str], bool, bool) -> Dict[str, List[bytes]]
+        # type: (str, list[str], bool, bool) -> dict[str, list[bytes]]
         """
         Return multiple attributes of a single LDAP object.
 
@@ -630,13 +616,12 @@ class access(object):
         :param bool required: Raise an exception instead of returning an empty dictionary.
         :param bool exceptions: Ignore.
         :returns: A dictionary mapping the requested attributes to a list of their values.
-        :rtype: dict[str, list[str]]
         :raises ldap.NO_SUCH_OBJECT: If the LDAP object is not accessible.
         """
         return self.lo.get(dn, attr, required)
 
     def getAttr(self, dn, attr, required=False, exceptions=False):
-        # type: (str, str, bool, bool) -> List[bytes]
+        # type: (str, str, bool, bool) -> list[bytes]
         """
         Return a single attribute of a single LDAP object.
 
@@ -645,13 +630,12 @@ class access(object):
         :param bool required: Raise an exception instead of returning an empty dictionary.
         :param bool exceptions: Ignore.
         :returns: A list of values.
-        :rtype: list[bytes]
         :raises ldap.NO_SUCH_OBJECT: If the LDAP object is not accessible.
         """
         return self.lo.getAttr(dn, attr, required)
 
     def search(self, filter=u'(objectClass=*)', base=u'', scope=u'sub', attr=[], unique=False, required=False, timeout=-1, sizelimit=0, serverctrls=None, response=None):
-        # type: (str, str, str, List[str], bool, bool, int, int, Optional[List[ldap.controls.LDAPControl]], Optional[Dict[str, ldap.controls.LDAPControl]]) -> List[Tuple[str, Dict[str, List[bytes]]]]
+        # type: (str, str, str, list[str], bool, bool, int, int, list[ldap.controls.LDAPControl] | None, dict[str, ldap.controls.LDAPControl] | None) -> list[tuple[str, dict[str, list[bytes]]]]
         """
         Perform LDAP search and return values.
 
@@ -659,16 +643,13 @@ class access(object):
         :param str base: the starting point for the search.
         :param str scope: Specify the scope of the search to be one of `base`, `base+one`, `one`, `sub`, or `domain` to specify a base object, base plus one-level, one-level, subtree, or children search.
         :param attr: The list of attributes to fetch.
-        :type attr: list[str]
         :param bool unique: Raise an exception if more than one object matches.
         :param bool required: Raise an exception instead of returning an empty dictionary.
         :param int timeout: wait at most `timeout` seconds for a search to complete. `-1` for no limit.
         :param int sizelimit: retrieve at most `sizelimit` entries for a search. `0` for no limit.
         :param serverctrls: a list of ldap.controls.LDAPControl instances sent to the server along with the LDAP request
-        :type serverctrls: list[ldap.controls.LDAPControl]
         :param dict response: An optional dictionary to receive the server controls of the result.
         :returns: A list of 2-tuples (dn, values) for each LDAP object, where values is a dictionary mapping attribute names to a list of values.
-        :rtype: list[tuple[str, dict[str, list[str]]]]
         :raises univention.admin.uexceptions.noObject: Indicates the target object cannot be found.
         :raises univention.admin.uexceptions.insufficientInformation: Indicates that the matching rule specified in the search filter does not match a rule defined for the attribute's syntax.
         :raises univention.admin.uexceptions.ldapTimeout: Indicates that the time limit of the LDAP client was exceeded while waiting for a result.
@@ -695,7 +676,7 @@ class access(object):
             raise univention.admin.uexceptions.ldapError(_err2str(msg), original_exception=msg)
 
     def searchDn(self, filter=u'(objectClass=*)', base=u'', scope=u'sub', unique=False, required=False, timeout=-1, sizelimit=0, serverctrls=None, response=None):
-        # type: (str, str, str, bool, bool, int, int, Optional[List[ldap.controls.LDAPControl]], Optional[Dict[str, ldap.controls.LDAPControl]]) -> List[str]
+        # type: (str, str, str, bool, bool, int, int, list[ldap.controls.LDAPControl] | None, dict[str, ldap.controls.LDAPControl] | None) -> list[str]
         """
         Perform LDAP search and return distinguished names only.
 
@@ -707,10 +688,8 @@ class access(object):
         :param int timeout: wait at most timeout seconds for a search to complete. `-1` for no limit.
         :param int sizelimit: retrieve at most sizelimit entries for a search. `0` for no limit.
         :param serverctrls: a list of :py:class:`ldap.controls.LDAPControl` instances sent to the server along with the LDAP request.
-        :type serverctrls: list[ldap.controls.LDAPControl]
         :param dict response: An optional dictionary to receive the server controls of the result.
         :returns: A list of distinguished names.
-        :rtype: list[str]
         :raises univention.admin.uexceptions.noObject: Indicates the target object cannot be found.
         :raises univention.admin.uexceptions.insufficientInformation: Indicates that the matching rule specified in the search filter does not match a rule defined for the attribute's syntax.
         :raises univention.admin.uexceptions.ldapTimeout: Indicates that the time limit of the LDAP client was exceeded while waiting for a result.
@@ -737,7 +716,7 @@ class access(object):
             raise univention.admin.uexceptions.ldapError(_err2str(msg), original_exception=msg)
 
     def getPolicies(self, dn, policies=None, attrs=None, result=None, fixedattrs=None):
-        # type: (str, Optional[List[str]], Optional[Dict[str, List[Any]]], Any, Any) -> Dict[str, Dict[str, Any]]
+        # type: (str, list[str] | None, dict[str, list[Any]] | None, Any, Any) -> dict[str, dict[str, Any]]
         """
         Return |UCS| policies for |LDAP| entry.
 
@@ -752,7 +731,7 @@ class access(object):
         return self.lo.getPolicies(dn, policies, attrs, result, fixedattrs)
 
     def add(self, dn, al, exceptions=False, serverctrls=None, response=None, ignore_license=False):
-        # type: (str, List[Tuple[str, Any]], bool, Optional[List[ldap.controls.LDAPControl]], Optional[Dict]) -> None
+        # type: (str, list[tuple[str, Any]], bool, list[ldap.controls.LDAPControl] | None, dict | None, bool) -> None
         """
         Add LDAP entry at distinguished name and attributes in add_list=(attribute-name, old-values. new-values) or (attribute-name, new-values).
 
@@ -761,7 +740,6 @@ class access(object):
         :param bool exceptions: Raise the low level exception instead of the wrapping UDM exceptions.
         :param serverctrls: a list of ldap.controls.LDAPControl instances sent to the server along with the LDAP request
         :param bool ignore_license: Ignore license check if True.
-        :type serverctrls: list[ldap.controls.LDAPControl]
         :param dict response: An optional dictionary to receive the server controls of the result.
         :raises univention.admin.uexceptions.licenseDisableModify: if the UCS licence prohibits any modificcation
         :raises univention.admin.uexceptions.objectExists: if the LDAP object already exists.
@@ -791,7 +769,7 @@ class access(object):
             raise univention.admin.uexceptions.ldapError(_err2str(msg), original_exception=msg)
 
     def modify(self, dn, changes, exceptions=False, ignore_license=False, serverctrls=None, response=None, rename_callback=None):
-        # type: (str, List[Tuple[str, Any, Any]], bool, int, Optional[List[ldap.controls.LDAPControl]], Optional[Dict], Optional[Callable]) -> str
+        # type: (str, list[tuple[str, Any, Any]], bool, int, list[ldap.controls.LDAPControl] | None, dict | None, Callable | None) -> str
         """
         Modify LDAP entry DN with attributes in changes=(attribute-name, old-values, new-values).
 
@@ -800,10 +778,8 @@ class access(object):
         :param bool exceptions: Raise the low level exception instead of the wrapping UDM exceptions.
         :param bool ignore_license: Ignore license check if True.
         :param serverctrls: a list of ldap.controls.LDAPControl instances sent to the server along with the LDAP request
-        :type serverctrls: list[ldap.controls.LDAPControl]
         :param dict response: An optional dictionary to receive the server controls of the result.
         :returns: The distinguished name.
-        :rtype: str
         """
         self._validateLicense()
         if not self.allow_modify and not ignore_license:
@@ -827,19 +803,18 @@ class access(object):
             raise univention.admin.uexceptions.ldapError(_err2str(msg), original_exception=msg)
 
     def rename(self, dn, newdn, move_childs=0, ignore_license=False, serverctrls=None, response=None):
-        # type: (str, str, int, bool, Optional[List[ldap.controls.LDAPControl]], Optional[Dict]) -> None
+        # type: (str, str, int, bool, list[ldap.controls.LDAPControl] | None, dict | None) -> None
         """
         Rename a LDAP object.
 
-        :param str dn: The old distinguished name of the object to rename.
-        :param str newdn: The new distinguished name of the object to rename.
-        :param int move_childs: Also rename the sub children. Must be `0` always as `1` is not implemented.
-        :param bool ignore_license: Ignore license check if True.
+        :param dn: The old distinguished name of the object to rename.
+        :param newdn: The new distinguished name of the object to rename.
+        :param move_childs: Also rename the sub children. Must be `0` always as `1` is not implemented.
+        :param ignore_license: Ignore license check if True.
         :param serverctrls: a list of ldap.controls.LDAPControl instances sent to the server along with the LDAP request
-        :type serverctrls: list[ldap.controls.LDAPControl]
-        :param dict response: An optional dictionary to receive the server controls of the result.
+        :param response: An optional dictionary to receive the server controls of the result.
         """
-        if not move_childs == 0:
+        if move_childs != 0:
             raise univention.admin.uexceptions.noObject(_("Moving children is not supported."))
         self._validateLicense()
         if not self.allow_modify and not ignore_license:
@@ -894,24 +869,22 @@ class access(object):
             raise univention.admin.uexceptions.ldapError(_err2str(msg), original_exception=msg)
 
     def parentDn(self, dn):
-        # type: (str) -> Optional[str]
+        # type: (str) -> str | None
         """
         Return the parent container of a distinguished name.
 
         :param str dn: The distinguished name.
         :return: The parent distinguished name or None if the LDAP base is reached.
-        :rtype: str or None
         """
         return self.lo.parentDn(dn)
 
     def explodeDn(self, dn, notypes=False):
-        # type: (str, bool) -> List[str]
+        # type: (str, bool) -> list[str]
         """
         Break up a DN into its component parts.
 
         :param str dn: The distinguished name.
         :param bool notypes: Return only the component's attribute values if True. Also the attribute types if False.
         :return: A list of relative distinguished names.
-        :rtype: list[str]
         """
         return self.lo.explodeDn(dn, notypes)
